package com.itbz.controller;

import com.itbz.service.PaymentService;
import io.github.resilience4j.circuitbreaker.annotation.CircuitBreaker;
import io.github.resilience4j.retry.annotation.Retry;
import io.github.resilience4j.timelimiter.annotation.TimeLimiter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.concurrent.CompletableFuture;
import java.util.function.Supplier;

@RestController
@RequestMapping("/order")
@Slf4j
public class OrderController {
    @Autowired
    private PaymentService paymentService;

    /**
     * 测试 超时降级
     * @return
     */
    @GetMapping("/timeout")
    @TimeLimiter(name = "a",fallbackMethod = "timeoutFallback")
    public CompletableFuture<String> timeout(){
        log.info("方法进入");
        CompletableFuture<String> stringCompletableFuture = CompletableFuture.supplyAsync((Supplier<String>) () -> (paymentService.timeout()));
        log.info("方法结束");
        return stringCompletableFuture;
    }

    /**
     * 超时降级方法
     */
    public CompletableFuture<String> timeoutFallback(Exception e){
        e.printStackTrace();
        return CompletableFuture.completedFuture("超时了");
    }

    /**
     * 测试重试机制
     */
    @GetMapping("/retry")
    @Retry(name = "aa")
    public CompletableFuture<String> retry(){
        log.info("******* 进入方法 *******");
        CompletableFuture<String> stringCompletableFuture = CompletableFuture.supplyAsync((Supplier<String>) () -> (paymentService.index()));
        log.info("******* 离开方法 *******");
        return stringCompletableFuture;
    }

    /**
     * 异常比例熔断降级
     * @return
     */
    @GetMapping("/circuitbreaker")
    @CircuitBreaker(name = "backendA",fallbackMethod = "circuitbreakerFallbake")
    public String circuitbreakerTest(){
        log.info("******* 进入方法 *******");
        String retry = paymentService.index();
        log.info("******* 离开方法 *******");
        return retry;
    }

    /**
     * 异常比例熔断降级方法
     * @param e
     * @return
     */
    public String circuitbreakerFallbake(Exception e){
        e.printStackTrace();
        return "服务繁忙，请稍后再试";
    }

    /**
     * 慢调用熔断降级
     * @return
     */
    @GetMapping("/slowcircuitbackend")
    @CircuitBreaker(name = "backendB",fallbackMethod = "slowcircuitbreakerFallback")
    public String slowcircuitbreaker(){
        log.info("******* 进入方法 *******");
        String index = paymentService.index();
        log.info("******* 离开方法 *******");
        return index;
    }

    /**
     * 慢调用熔断降级方法
     * @param e
     * @return
     */
    public String slowcircuitbreakerFallback(Exception e){
        e.printStackTrace();
        return "太慢了......";
    }


}
